import numpy as np
import scipy.interpolate
from . import sph_coll_eqns as sc
from . import growth_eqns as growth
from diffeqpy import de

def SphEvoGR(dini,Omega_m,xi,xf):
    yi = [0.,-dini/3.]
    x_eval = (xi,xf)   
    params = [Omega_m,dini,xi]
    ode = de.ODEProblem(sc.jul_rhs_gr, yi, x_eval, params)
    sol = de.solve(ode, save_everystep=False, save_start=False, abstol=1e-6, reltol=1e-6)
    return sol.u[0][0]

def calc_tau_gr(dini_vec,dlin_vec,Omega_m,xi,xf):
    dNL_vec = np.zeros(dlin_vec.shape)
    for j, dini in enumerate(dini_vec):
        yf = SphEvoGR(dini,Omega_m,xi,xf)
        dNL_vec[j] = (1 + dini) / (np.exp(xi)/np.exp(xf) * yf + 1)**3

    return scipy.interpolate.CubicSpline(dNL_vec,dlin_vec)

def calc_growth_fr_full(Omega_m,fR0,n,k,xf):
    gi = [1.,0.]
    xi = np.log(1e-10)
    x_eval = (xi,xf)   
    params = [Omega_m,fR0,n,k]
    ode_growth = de.ODEProblem(growth.jul_rhs_growth_fr, gi, x_eval, params)
    sol_growth = de.solve(ode_growth, save_everystep=True, abstol=1e-8, reltol=1e-8)
    return sol_growth

def calc_growth_dgp_full(Omega_m,rcH0,xf):
    gi = [1.,0.]
    xi = np.log(1e-10)
    x_eval = (xi,xf)   
    params = [Omega_m,rcH0]
    ode_growth = de.ODEProblem(growth.jul_rhs_growth_dgp, gi, x_eval, params)
    sol_growth = de.solve(ode_growth, save_everystep=True, abstol=1e-8, reltol=1e-8)
    return sol_growth


import numpy as np
from julia import Main

def E(a, Omega_m, w0, wa):
    return np.sqrt(Omega_m/a**3. + (1.-Omega_m) * a**(-3.*(1.+w0+wa)) * np.exp(-3*wa*(1.-a)))

def growth_int(a, Omega_m, w0, wa, ai, af):
    return 2.5*Omega_m*E(af,Omega_m,w0,wa)/ai*1/(a*E(a,Omega_m,w0,wa))**3

jul_rhs_growth_fr = Main.eval("""
    function growth_fr(dy,y,p,x)
        Om0, fR0, n, k = p
        Omega_m = Om0/(Om0 + exp(3*x)*(1-Om0))
        Omega_L = 1 - Omega_m
        m2 = (1/2997.92458)^2 * (Om0*exp(-3*x) + 4*(1-Om0))^(n+2)/(abs(fR0)*(n+1)*(4-3*Om0)^(n+1))
        eps = k^2/(3 * (k^2 + m2*exp(2*x)))

        dy[1] = y[2]
        dy[2] = -(2.5 + 1.5*Omega_L)*y[2] - 1.5*(2*Omega_L - Omega_m*eps)*y[1]
    end
    """)

jul_rhs_growth_dgp = Main.eval("""
    function growth_dgp(dy,y,p,x)
        Om0, rcH0 = p
        Omega_m = Om0/(Om0 + exp(3*x)*(1-Om0))
        Omega_L = 1 - Omega_m
        Om3x = Om0 * exp(-3*x)
        beta = 1 + 2*rcH0 * sqrt(Om3x + (1-Om0)) * (1 - Om3x/(2*(Om3x + (1-Om0))))
        eps = 1/(3*beta)

        dy[1] = y[2]
        dy[2] = -(2.5 + 1.5*Omega_L)*y[2] - 1.5*(2*Omega_L - Omega_m*eps)*y[1]
    end
    """)